class KctKilometraz < ActiveRecord::Base 
  belongs_to :kct_trail
  belongs_to :kct_guidepost
  
  delegate :kctref, :guidepost, to: :kct_guidepost
  delegate :trail, to: :kct_trail
  delegate :osm_trail, to: :trail, :allow_nil => true
  
  self.table_name = 'kct_kilometraz'
  
  # estimates my position on trail in kms by averaging distance from osm signs with known osm km values
  def estimated_km_from_osm_kms
    kct_master_signs = self.kct_trail.kct_signs.select {|ks| !ks.sign.nil? and !ks.sign.osm_guidepost.nil?}  # these are the other signs on trail which i will be using to guess the position
    estimated_osm_kms = kct_master_signs.collect do |kct_master_sign|
      if osm_master_sign = kct_master_sign.sign.osm_sign
        osm_master_sign.km + self.kct_trail.trail_relative_distance(kct_master_sign,self)
      else
        nil
      end
    end
    estimated_osm_kms.compact!
    result = estimated_osm_kms.inject(0.0) { |sum, el| sum + el } / estimated_osm_kms.size
    result = 1 if result<0
    return result
  end
  
  def estimated_coordinates_from_osm
    osm_trail.gpx.point_with_kilometraz(estimated_km_from_osm_kms)
#     osm_trail.andand.point_with_kilometraz(estimated_km_from_osm_kms)
  end

  def sign
    if guidepost.nil? or trail.nil?
       nil
    else
      Sign.where(:guidepost_id=>guidepost.id, :trail_id=>trail.id).first
    end
  end

  def exists?; !sign.nil? end
  
end
